@@ -11,7 +11,7 @@ import shutil
 
 # local imports
 from dotdrop.logger import Logger
-from dotdrop.templategen import Templategen
+from dotdrop.linktypes import LinkTypes
 import dotdrop.utils as utils
 from dotdrop.exceptions import UndefinedException
 
@@ -22,7 +22,7 @@ class Installer:
                  dry=False, safe=False, workdir='~/.config/dotdrop',
                  debug=False, diff=True, totemp=None, showdiff=False,
                  backup_suffix='.dotdropbak', diff_cmd=''):
-        """constructor
+        """
         @base: directory path where to search for templates
         @create: create directory hierarchy if missing when installing
         @backup: backup existing dotfile when installing
@@ -40,7 +40,11 @@ class Installer:
         self.backup = backup
         self.dry = dry
         self.safe = safe
-        self.workdir = os.path.expanduser(workdir)
+        workdir = os.path.expanduser(workdir)
+        workdir = os.path.normpath(workdir)
+        self.workdir = workdir
+        base = os.path.expanduser(base)
+        base = os.path.normpath(base)
         self.base = base
         self.debug = debug
         self.diff = diff
@@ -48,34 +52,33 @@ class Installer:
         self.showdiff = showdiff
         self.backup_suffix = backup_suffix
         self.diff_cmd = diff_cmd
-        self.comparing = False
         self.action_executed = False
+        # avoids printing file copied logs
+        # when using install_to_tmp for comparing
+        self.comparing = False
+
         self.log = Logger()
 
-    def _log_install(self, boolean, err):
-        if not self.debug:
-            return boolean, err
-        if boolean:
-            self.log.dbg('install: SUCCESS')
-        else:
-            if err:
-                self.log.dbg('install: ERROR: {}'.format(err))
-            else:
-                self.log.dbg('install: IGNORED')
-        return boolean, err
+    ########################################################
+    # public methods
+    ########################################################
 
-    def install(self, templater, src, dst,
+    def install(self, templater, src, dst, linktype,
                 actionexec=None, noempty=False,
-                ignore=[], template=True):
+                ignore=[], is_template=True,
+                chmod=None):
         """
-        install src to dst using a template
+        install src to dst
+
         @templater: the templater object
         @src: dotfile source path in dotpath
         @dst: dotfile destination path in the FS
+        @linktype: linktypes.LinkTypes
         @actionexec: action executor callback
         @noempty: render empty template flag
         @ignore: pattern to ignore when installing
-        @template: template this dotfile
+        @is_template: this dotfile is a template
+        @chmod: rights to apply if any
 
         return
         - True, None        : success
@@ -83,126 +86,200 @@ class Installer:
         - False, None       : ignored
         """
         if self.debug:
-            self.log.dbg('installing \"{}\" to \"{}\"'.format(src, dst))
-        if not dst or not src:
-            if self.debug:
-                self.log.dbg('empty dst for {}'.format(src))
-            return self._log_install(True, None)
-        self.action_executed = False
-        src = os.path.join(self.base, os.path.expanduser(src))
+            msg = 'installing \"{}\" to \"{}\" (link: {})'
+            self.log.dbg(msg.format(src, dst, str(linktype)))
+        src, dst, cont, err = self._check_paths(src, dst, chmod)
+        if not cont:
+            return self._log_install(cont, err)
+
+        # check source file exists
+        src = os.path.join(self.base, src)
         if not os.path.exists(src):
             err = 'source dotfile does not exist: {}'.format(src)
             return self._log_install(False, err)
-        dst = os.path.expanduser(dst)
+
+        self.action_executed = False
+
+        # install to temporary dir
+        # and ignore any actions
         if self.totemp:
-            dst = self._pivot_path(dst, self.totemp)
-        if utils.samefile(src, dst):
-            # symlink loop
-            err = 'dotfile points to itself: {}'.format(dst)
-            return self._log_install(False, err)
+            r, err, _ = self.install_to_temp(templater, self.totemp,
+                                             src, dst, is_template=is_template,
+                                             chmod=chmod)
+            return self._log_install(r, err)
+
         isdir = os.path.isdir(src)
         if self.debug:
             self.log.dbg('install {} to {}'.format(src, dst))
-            self.log.dbg('is a directory \"{}\": {}'.format(src, isdir))
-        if isdir:
-            b, e = self._install_dir(templater, src, dst,
-                                     actionexec=actionexec,
-                                     noempty=noempty, ignore=ignore,
-                                     template=template)
-            return self._log_install(b, e)
-        b, e = self._install_file(templater, src, dst,
-                                  actionexec=actionexec,
-                                  noempty=noempty, ignore=ignore,
-                                  template=template)
-        return self._log_install(b, e)
+            self.log.dbg('\"{}\" is a directory: {}'.format(src, isdir))
 
-    def link(self, templater, src, dst, actionexec=None, template=True):
+        if linktype == LinkTypes.NOLINK:
+            # normal file
+            if isdir:
+                r, err = self._copy_dir(templater, src, dst,
+                                        actionexec=actionexec,
+                                        noempty=noempty, ignore=ignore,
+                                        is_template=is_template,
+                                        chmod=chmod)
+            else:
+                r, err = self._copy_file(templater, src, dst,
+                                         actionexec=actionexec,
+                                         noempty=noempty, ignore=ignore,
+                                         is_template=is_template,
+                                         chmod=chmod)
+        elif linktype == LinkTypes.LINK:
+            # symlink
+            r, err = self._link(templater, src, dst,
+                                actionexec=actionexec,
+                                is_template=is_template)
+        elif linktype == LinkTypes.LINK_CHILDREN:
+            # symlink direct children
+            if not isdir:
+                if self.debug:
+                    msg = 'symlink children of {} to {}'
+                    self.log.dbg(msg.format(src, dst))
+                err = 'source dotfile is not a directory: {}'.format(src)
+                r = False
+            else:
+                r, err = self._link_children(templater, src, dst,
+                                             actionexec=actionexec,
+                                             is_template=is_template)
+
+        if self.debug:
+            self.log.dbg('before chmod: {} err:{}'.format(r, err))
+
+        if self.dry:
+            return self._log_install(r, err)
+
+        # handle chmod
+        # - on success (r, not err)
+        # - no change (not r, not err)
+        # but not when
+        # - error (not r, err)
+        # - aborted (not r, err)
+        if (r or (not r and not err)):
+            if not chmod:
+                chmod = utils.get_file_perm(src)
+            dstperms = utils.get_file_perm(dst)
+            if dstperms != chmod:
+                # apply mode
+                msg = 'chmod {} to {:o}'.format(dst, chmod)
+                if self.safe and not self.log.ask(msg):
+                    r = False
+                    err = 'aborted'
+                else:
+                    if not self.comparing:
+                        self.log.sub('chmod {} to {:o}'.format(dst, chmod))
+                    if utils.chmod(dst, chmod, debug=self.debug):
+                        r = True
+                    else:
+                        r = False
+                        err = 'chmod failed'
+
+        return self._log_install(r, err)
+
+    def install_to_temp(self, templater, tmpdir, src, dst,
+                        is_template=True, chmod=None):
         """
-        set src as the link target of dst
-        @templater: the templater
+        install a dotfile to a tempdir
+
+        @templater: the templater object
+        @tmpdir: where to install
         @src: dotfile source path in dotpath
         @dst: dotfile destination path in the FS
-        @actionexec: action executor callback
-        @template: template this dotfile
+        @is_template: this dotfile is a template
+        @chmod: rights to apply if any
+
+        return
+        - success, error-if-any, dotfile-installed-path
+        """
+        if self.debug:
+            self.log.dbg('tmp install {} (defined dst: {})'.format(src, dst))
+        src, dst, cont, err = self._check_paths(src, dst, chmod)
+        if not cont:
+            return self._log_install(cont, err)
+
+        ret = False
+        tmpdst = ''
+
+        # save flags
+        self.comparing = True
+        drysaved = self.dry
+        self.dry = False
+        diffsaved = self.diff
+        self.diff = False
+        createsaved = self.create
+        self.create = True
+        totemp = self.totemp
+        self.totemp = None
+
+        # install the dotfile to a temp directory
+        tmpdst = self._pivot_path(dst, tmpdir)
+        ret, err = self.install(templater, src, tmpdst,
+                                LinkTypes.NOLINK,
+                                is_template=is_template,
+                                chmod=chmod)
+        if self.debug:
+            if ret:
+                self.log.dbg('tmp installed in {}'.format(tmpdst))
+
+        # restore flags
+        self.dry = drysaved
+        self.diff = diffsaved
+        self.create = createsaved
+        self.comparing = False
+        self.totemp = totemp
+
+        return ret, err, tmpdst
+
+    ########################################################
+    # low level accessors for public methods
+    ########################################################
+
+    def _link(self, templater, src, dst, actionexec=None, is_template=True):
+        """
+        install link:link
 
         return
         - True, None        : success
         - False, error_msg  : error
         - False, None       : ignored
+        - False, 'aborted'    : user aborted
         """
-        if self.debug:
-            self.log.dbg('link \"{}\" to \"{}\"'.format(src, dst))
-        if not dst or not src:
+        if is_template:
             if self.debug:
-                self.log.dbg('empty dst for {}'.format(src))
-            return self._log_install(True, None)
-        self.action_executed = False
-        src = os.path.normpath(os.path.join(self.base,
-                                            os.path.expanduser(src)))
-        if not os.path.exists(src):
-            err = 'source dotfile does not exist: {}'.format(src)
-            return self._log_install(False, err)
-        dst = os.path.normpath(os.path.expanduser(dst))
-        if self.totemp:
-            # ignore actions
-            b, e = self.install(templater, src, dst, actionexec=None,
-                                template=template)
-            return self._log_install(b, e)
-
-        if template and Templategen.is_template(src):
-            if self.debug:
-                self.log.dbg('dotfile is a template')
-                self.log.dbg('install to {} and symlink'.format(self.workdir))
+                self.log.dbg('is a template')
+                self.log.dbg('install to {}'.format(self.workdir))
             tmp = self._pivot_path(dst, self.workdir, striphome=True)
-            i, err = self.install(templater, src, tmp, actionexec=actionexec,
-                                  template=template)
-            if not i and not os.path.exists(tmp):
-                return self._log_install(i, err)
+            r, err = self.install(templater, src, tmp,
+                                  LinkTypes.NOLINK,
+                                  actionexec=actionexec,
+                                  is_template=is_template)
+            if not r and not os.path.exists(tmp):
+                return r, err
             src = tmp
-        b, e = self._link(src, dst, actionexec=actionexec)
-        return self._log_install(b, e)
+        r, err = self._symlink(src, dst, actionexec=actionexec)
+        return r, err
 
-    def link_children(self, templater, src, dst, actionexec=None,
-                      template=True):
+    def _link_children(self, templater, src, dst,
+                       actionexec=None, is_template=True):
         """
-        link all files under a given directory
-        @templater: the templater
-        @src: dotfile source path in dotpath
-        @dst: dotfile destination path in the FS
-        @actionexec: action executor callback
-        @template: template this dotfile
+        install link:link_children
 
         return
-        - True, None: success
-        - False, error_msg: error
-        - False, None, ignored
+        - True, None        : success
+        - False, error_msg  : error
+        - False, None       : ignored
+        - False, 'aborted'    : user aborted
         """
-        if self.debug:
-            self.log.dbg('link_children \"{}\" to \"{}\"'.format(src, dst))
-        if not dst or not src:
-            if self.debug:
-                self.log.dbg('empty dst for {}'.format(src))
-            return self._log_install(True, None)
-        self.action_executed = False
-        parent = os.path.join(self.base, os.path.expanduser(src))
-
-        # Fail if source doesn't exist
-        if not os.path.exists(parent):
-            err = 'source dotfile does not exist: {}'.format(parent)
-            return self._log_install(False, err)
-
-        # Fail if source not a directory
-        if not os.path.isdir(parent):
-            if self.debug:
-                self.log.dbg('symlink children of {} to {}'.format(src, dst))
-
-            err = 'source dotfile is not a directory: {}'.format(parent)
-            return self._log_install(False, err)
-
-        dst = os.path.normpath(os.path.expanduser(dst))
+        parent = os.path.join(self.base, src)
         if not os.path.lexists(dst):
-            self.log.sub('creating directory "{}"'.format(dst))
-            os.makedirs(dst)
+            if self.dry:
+                self.log.dry('would create directory "{}"'.format(dst))
+            else:
+                if not self.comparing:
+                    self.log.sub('creating directory "{}"'.format(dst))
+                self._create_dirs(dst)
 
         if os.path.isfile(dst):
             msg = ''.join([
@@ -211,10 +288,9 @@ class Installer:
             ]).format(dst)
 
             if self.safe and not self.log.ask(msg):
-                err = 'ignoring "{}", nothing installed'.format(dst)
-                return self._log_install(False, err)
+                return False, 'aborted'
             os.unlink(dst)
-            os.mkdir(dst)
+            self._create_dirs(dst)
 
         children = os.listdir(parent)
         srcs = [os.path.normpath(os.path.join(parent, child))
@@ -224,25 +300,27 @@ class Installer:
 
         installed = 0
         for i in range(len(children)):
-            src = srcs[i]
-            dst = dsts[i]
+            subsrc = srcs[i]
+            subdst = dsts[i]
 
             if self.debug:
-                self.log.dbg('symlink child {} to {}'.format(src, dst))
+                self.log.dbg('symlink child {} to {}'.format(subsrc, subdst))
 
-            if template and Templategen.is_template(src):
+            if is_template:
                 if self.debug:
-                    self.log.dbg('dotfile is a template')
+                    self.log.dbg('child is a template')
                     self.log.dbg('install to {} and symlink'
                                  .format(self.workdir))
-                tmp = self._pivot_path(dst, self.workdir, striphome=True)
-                r, e = self.install(templater, src, tmp, actionexec=actionexec,
-                                    template=template)
+                tmp = self._pivot_path(subdst, self.workdir, striphome=True)
+                r, e = self.install(templater, subsrc, tmp,
+                                    LinkTypes.NOLINK,
+                                    actionexec=actionexec,
+                                    is_template=is_template)
                 if not r and e and not os.path.exists(tmp):
                     continue
-                src = tmp
+                subsrc = tmp
 
-            ret, err = self._link(src, dst, actionexec=actionexec)
+            ret, err = self._symlink(subsrc, subdst, actionexec=actionexec)
             if ret:
                 installed += 1
                 # void actionexec if dotfile installed
@@ -250,18 +328,23 @@ class Installer:
                 actionexec = None
             else:
                 if err:
-                    return self._log_install(ret, err)
+                    return ret, err
 
-        return self._log_install(installed > 0, None)
+        return installed > 0, None
 
-    def _link(self, src, dst, actionexec=None):
+    ########################################################
+    # file operations
+    ########################################################
+
+    def _symlink(self, src, dst, actionexec=None):
         """
         set src as a link target of dst
 
         return
-        - True, None: success
-        - False, error_msg: error
-        - False, None, ignored
+        - True, None        : success
+        - False, error_msg  : error
+        - False, None       : ignored
+        - False, 'aborted'    : user aborted
         """
         overwrite = not self.safe
         if os.path.lexists(dst):
@@ -274,11 +357,10 @@ class Installer:
                 self.log.dry('would remove {} and link to {}'.format(dst, src))
                 return True, None
             if self.showdiff:
-                self._diff_before_write(src, dst, quiet=False)
+                self._show_diff_before_write(src, dst)
             msg = 'Remove "{}" for link creation?'.format(dst)
             if self.safe and not self.log.ask(msg):
-                err = 'ignoring "{}", link was not created'.format(dst)
-                return False, err
+                return False, 'aborted'
             overwrite = True
             try:
                 utils.removepath(dst)
@@ -299,41 +381,49 @@ class Installer:
         if os.path.lexists(dst):
             msg = 'Remove "{}" for link creation?'.format(dst)
             if self.safe and not overwrite and not self.log.ask(msg):
-                err = 'ignoring "{}", link was not created'.format(dst)
-                return False, err
+                return False, 'aborted'
             try:
                 utils.removepath(dst)
             except OSError as e:
                 err = 'something went wrong with {}: {}'.format(src, e)
                 return False, err
         os.symlink(src, dst)
-        self.log.sub('linked {} to {}'.format(dst, src))
+        if not self.comparing:
+            self.log.sub('linked {} to {}'.format(dst, src))
         return True, None
 
-    def _get_tmp_file_vars(self, src, dst):
-        tmp = {}
-        tmp['_dotfile_sub_abs_src'] = src
-        tmp['_dotfile_sub_abs_dst'] = dst
-        return tmp
+    def _copy_file(self, templater, src, dst,
+                   actionexec=None, noempty=False,
+                   ignore=[], is_template=True,
+                   chmod=None):
+        """
+        install src to dst when is a file
 
-    def _install_file(self, templater, src, dst,
-                      actionexec=None, noempty=False,
-                      ignore=[], template=True):
-        """install src to dst when is a file"""
+        return
+        - True, None        : success
+        - False, error_msg  : error
+        - False, None       : ignored
+        - False, 'aborted'    : user aborted
+        """
         if self.debug:
             self.log.dbg('deploy file: {}'.format(src))
             self.log.dbg('ignore empty: {}'.format(noempty))
             self.log.dbg('ignore pattern: {}'.format(ignore))
-            self.log.dbg('template: {}'.format(template))
+            self.log.dbg('is_template: {}'.format(is_template))
             self.log.dbg('no empty: {}'.format(noempty))
 
+        # check no loop
+        if utils.samefile(src, dst):
+            err = 'dotfile points to itself: {}'.format(dst)
+            return False, err
+
         if utils.must_ignore([src, dst], ignore, debug=self.debug):
             if self.debug:
                 self.log.dbg('ignoring install of {} to {}'.format(src, dst))
             return False, None
 
         if utils.samefile(src, dst):
-            # symlink loop
+            # loop
             err = 'dotfile points to itself: {}'.format(dst)
             return False, err
 
@@ -343,7 +433,7 @@ class Installer:
 
         # handle the file
         content = None
-        if template:
+        if is_template:
             # template the file
             saved = templater.add_tmp_vars(self._get_tmp_file_vars(src, dst))
             try:
@@ -352,6 +442,7 @@ class Installer:
                 return False, str(e)
             finally:
                 templater.restore_vars(saved)
+            # test is empty
             if noempty and utils.content_empty(content):
                 if self.debug:
                     self.log.dbg('ignoring empty template: {}'.format(src))
@@ -359,52 +450,53 @@ class Installer:
             if content is None:
                 err = 'empty template {}'.format(src)
                 return False, err
+
+        # write the file
         ret, err = self._write(src, dst,
                                content=content,
                                actionexec=actionexec,
-                               template=template)
-
-        # build return values
-        if ret < 0:
-            # error
-            return False, err
-        if ret > 0:
-            # already exists
-            if self.debug:
-                self.log.dbg('ignoring {}'.format(dst))
-            return False, None
-        if ret == 0:
-            # success
+                               chmod=chmod)
+        if ret and not err:
             if not self.dry and not self.comparing:
-                self.log.sub('copied {} to {}'.format(src, dst))
-            return True, None
-        # error
-        err = 'installing {} to {}'.format(src, dst)
-        return False, err
+                self.log.sub('install {} to {}'.format(src, dst))
+        return ret, err
 
-    def _install_dir(self, templater, src, dst,
-                     actionexec=None, noempty=False,
-                     ignore=[], template=True):
-        """install src to dst when is a directory"""
+    def _copy_dir(self, templater, src, dst,
+                  actionexec=None, noempty=False,
+                  ignore=[], is_template=True, chmod=None):
+        """
+        install src to dst when is a directory
+
+        return
+        - True, None        : success
+        - False, error_msg  : error
+        - False, None       : ignored
+        - False, 'aborted'    : user aborted
+        """
         if self.debug:
-            self.log.dbg('install dir {}'.format(src))
-            self.log.dbg('ignore empty: {}'.format(noempty))
+            self.log.dbg('deploy dir {}'.format(src))
         # default to nothing installed and no error
         ret = False, None
+
+        # create the directory anyway
         if not self._create_dirs(dst):
             err = 'creating directory for {}'.format(dst)
             return False, err
+
         # handle all files in dir
         for entry in os.listdir(src):
             f = os.path.join(src, entry)
+            if self.debug:
+                self.log.dbg('deploy sub from {}: {}'.format(dst, entry))
             if not os.path.isdir(f):
                 # is file
-                res, err = self._install_file(templater, f,
-                                              os.path.join(dst, entry),
-                                              actionexec=actionexec,
-                                              noempty=noempty,
-                                              ignore=ignore,
-                                              template=template)
+                res, err = self._copy_file(templater, f,
+                                           os.path.join(dst, entry),
+                                           actionexec=actionexec,
+                                           noempty=noempty,
+                                           ignore=ignore,
+                                           is_template=is_template,
+                                           chmod=None)
                 if not res and err:
                     # error occured
                     ret = res, err
@@ -414,12 +506,13 @@ class Installer:
                     ret = True, None
             else:
                 # is directory
-                res, err = self._install_dir(templater, f,
-                                             os.path.join(dst, entry),
-                                             actionexec=actionexec,
-                                             noempty=noempty,
-                                             ignore=ignore,
-                                             template=template)
+                res, err = self._copy_dir(templater, f,
+                                          os.path.join(dst, entry),
+                                          actionexec=actionexec,
+                                          noempty=noempty,
+                                          ignore=ignore,
+                                          is_template=is_template,
+                                          chmod=None)
                 if not res and err:
                     # error occured
                     ret = res, err
@@ -429,82 +522,67 @@ class Installer:
                     ret = True, None
         return ret
 
-    def _fake_diff(self, dst, content):
-        """
-        fake diff by comparing file content with content
-        returns True if same
-        """
-        cur = ''
-        with open(dst, 'br') as f:
-            cur = f.read()
-        return cur == content
-
     def _write(self, src, dst, content=None,
-               actionexec=None, template=True):
+               actionexec=None, chmod=None):
         """
         copy dotfile / write content to file
-        return  0, None:  for success,
-                1, None:  when already exists
-               -1, err: when error
-        content is always empty if template is False
-        and is to be ignored
+
+        return
+        - True, None        : success
+        - False, error_msg  : error
+        - False, None       : ignored
+        - False, 'aborted'    : user aborted
         """
         overwrite = not self.safe
         if self.dry:
             self.log.dry('would install {}'.format(dst))
-            return 0, None
+            return True, None
+
         if os.path.lexists(dst):
-            rights = os.stat(src).st_mode
-            samerights = False
             try:
-                samerights = os.stat(dst).st_mode == rights
+                os.stat(dst)
             except OSError as e:
                 if e.errno == errno.ENOENT:
                     # broken symlink
                     err = 'broken symlink {}'.format(dst)
-                    return -1, err
-            diff = None
+                    return False, err
+
+            src_mode = chmod
+            if not src_mode:
+                src_mode = utils.get_file_perm(src)
             if self.diff:
-                diff = self._diff_before_write(src, dst,
-                                               content=content,
-                                               quiet=True)
-                if not diff and samerights:
+                if not self._is_different(src, dst, content=content):
                     if self.debug:
                         self.log.dbg('{} is the same'.format(dst))
-                    return 1, None
+                    return False, None
             if self.safe:
                 if self.debug:
                     self.log.dbg('change detected for {}'.format(dst))
                 if self.showdiff:
-                    if diff is None:
-                        # get diff
-                        diff = self._diff_before_write(src, dst,
-                                                       content=content,
-                                                       quiet=True)
-                        if diff:
-                            self._print_diff(src, dst, diff)
+                    # get diff
+                    self._show_diff_before_write(src, dst,
+                                                 content=content)
                 if not self.log.ask('Overwrite \"{}\"'.format(dst)):
-                    self.log.warn('ignoring {}'.format(dst))
-                    return 1, None
+                    return False, 'aborted'
                 overwrite = True
         if self.backup and os.path.lexists(dst):
             self._backup(dst)
         base = os.path.dirname(dst)
         if not self._create_dirs(base):
             err = 'creating directory for {}'.format(dst)
-            return -1, err
+            return False, err
         r, e = self._exec_pre_actions(actionexec)
         if not r:
-            return -1, e
+            return False, e
         if self.debug:
-            self.log.dbg('install dotfile to \"{}\"'.format(dst))
+            self.log.dbg('install file to \"{}\"'.format(dst))
         # re-check in case action created the file
         if self.safe and not overwrite and os.path.lexists(dst):
             if not self.log.ask('Overwrite \"{}\"'.format(dst)):
                 self.log.warn('ignoring {}'.format(dst))
-                return 1, None
+                return False, 'aborted'
 
-        if template:
+        if content:
             # write content the file
             try:
                 with open(dst, 'wb') as f:
@@ -512,19 +590,44 @@ class Installer:
                 shutil.copymode(src, dst)
             except NotADirectoryError as e:
                 err = 'opening dest file: {}'.format(e)
-                return -1, err
+                return False, err
             except Exception as e:
-                return -1, str(e)
+                return False, str(e)
         else:
             # copy file
             try:
                 shutil.copyfile(src, dst)
                 shutil.copymode(src, dst)
             except Exception as e:
-                return -1, str(e)
-        return 0, None
+                return False, str(e)
+        return True, None
 
-    def _diff_before_write(self, src, dst, content=None, quiet=False):
+    ########################################################
+    # helpers
+    ########################################################
+
+    def _get_tmp_file_vars(self, src, dst):
+        tmp = {}
+        tmp['_dotfile_sub_abs_src'] = src
+        tmp['_dotfile_sub_abs_dst'] = dst
+        return tmp
+
+    def _is_different(self, src, dst, content=None):
+        """
+        returns True if file is different and
+        needs to be installed
+        """
+        # check file content
+        if content:
+            tmp = utils.write_to_tmpfile(content)
+            src = tmp
+        r = utils.fastdiff(src, dst)
+        if r:
+            if self.debug:
+                self.log.dbg('content differ')
+        return r
+
+    def _show_diff_before_write(self, src, dst, content=None):
         """
         diff before writing
         using a temp file if content is not None
@@ -534,12 +637,12 @@ class Installer:
         if content:
             tmp = utils.write_to_tmpfile(content)
             src = tmp
-        diff = utils.diff(modified=src, original=dst, raw=False,
+        diff = utils.diff(modified=src, original=dst,
                           diff_cmd=self.diff_cmd)
         if tmp:
             utils.removepath(tmp, logger=self.log)
 
-        if not quiet and diff:
+        if diff:
             self._print_diff(src, dst, diff)
         return diff
 
@@ -561,7 +664,10 @@ class Installer:
             return True
         if self.debug:
             self.log.dbg('mkdir -p {}'.format(directory))
-        os.makedirs(directory)
+        if not self.comparing:
+            self.log.sub('create directory {}'.format(directory))
+
+        os.makedirs(directory, exist_ok=True)
         return os.path.exists(directory)
 
     def _backup(self, path):
@@ -595,38 +701,36 @@ class Installer:
         self.action_executed = True
         return ret, err
 
-    def _install_to_temp(self, templater, src, dst, tmpdir, template=True):
-        """install a dotfile to a tempdir"""
-        tmpdst = self._pivot_path(dst, tmpdir)
-        r = self.install(templater, src, tmpdst, template=template)
-        return r, tmpdst
+    def _log_install(self, boolean, err):
+        """log installation process"""
+        if not self.debug:
+            return boolean, err
+        if boolean:
+            self.log.dbg('install: SUCCESS')
+        else:
+            if err:
+                self.log.dbg('install: ERROR: {}'.format(err))
+            else:
+                self.log.dbg('install: IGNORED')
+        return boolean, err
+
+    def _check_paths(self, src, dst, chmod):
+        """
+        check and normalize param
+        returns <src>, <dst>, <continue>, <error>
+        """
+        # check both path are valid
+        if not dst or not src:
+            err = 'empty dst or src for {}'.format(src)
+            if self.debug:
+                self.log.dbg(err)
+            return None, None, False, err
 
-    def install_to_temp(self, templater, tmpdir, src, dst, template=True):
-        """install a dotfile to a tempdir"""
-        ret = False
-        tmpdst = ''
-        # save some flags while comparing
-        self.comparing = True
-        drysaved = self.dry
-        self.dry = False
-        diffsaved = self.diff
-        self.diff = False
-        createsaved = self.create
-        self.create = True
         # normalize src and dst
         src = os.path.expanduser(src)
+        src = os.path.normpath(src)
+
         dst = os.path.expanduser(dst)
-        if self.debug:
-            self.log.dbg('tmp install {} (defined dst: {})'.format(src, dst))
-        # install the dotfile to a temp directory for comparing
-        r, tmpdst = self._install_to_temp(templater, src, dst, tmpdir,
-                                          template=template)
-        ret, err = r
-        if self.debug:
-            self.log.dbg('tmp installed in {}'.format(tmpdst))
-        # reset flags
-        self.dry = drysaved
-        self.diff = diffsaved
-        self.comparing = False
-        self.create = createsaved
-        return ret, err, tmpdst
+        dst = os.path.normpath(dst)
+
+        return src, dst, True, None
